@@ -5,7 +5,6 @@
 //! <https://hyperandroid.com/2020/02/12/v8-inspector-from-an-embedder-standpoint/>
 
 use crate::error::generic_error;
-use crate::error::AnyError;
 use crate::futures::channel::mpsc;
 use crate::futures::channel::mpsc::UnboundedReceiver;
 use crate::futures::channel::mpsc::UnboundedSender;
@@ -22,6 +21,7 @@ use crate::futures::task::Poll;
 use crate::serde_json;
 use crate::serde_json::json;
 use crate::serde_json::Value;
+use anyhow::Error;
 use parking_lot::Mutex;
 use std::cell::BorrowMutError;
 use std::cell::RefCell;
@@ -637,7 +637,7 @@ impl LocalInspectorSession {
     &mut self,
     method: &str,
     params: Option<serde_json::Value>,
-  ) -> Result<serde_json::Value, AnyError> {
+  ) -> Result<serde_json::Value, Error> {
     let id = self.next_message_id;
     self.next_message_id += 1;
 
