@@ -2,10 +2,10 @@
 
 use crate::bindings;
 use crate::error::generic_error;
-use crate::error::AnyError;
 use crate::module_specifier::ModuleSpecifier;
 use crate::runtime::exception_to_err_result;
 use crate::OpState;
+use anyhow::Error;
 use futures::future::FutureExt;
 use futures::stream::FuturesUnordered;
 use futures::stream::Stream;
@@ -54,9 +54,8 @@ pub struct ModuleSource {
 }
 
 pub type PrepareLoadFuture =
-  dyn Future<Output = (ModuleLoadId, Result<RecursiveModuleLoad, AnyError>)>;
-pub type ModuleSourceFuture =
-  dyn Future<Output = Result<ModuleSource, AnyError>>;
+  dyn Future<Output = (ModuleLoadId, Result<RecursiveModuleLoad, Error>)>;
+pub type ModuleSourceFuture = dyn Future<Output = Result<ModuleSource, Error>>;
 
 pub trait ModuleLoader {
   /// Returns an absolute URL.
@@ -71,7 +70,7 @@ pub trait ModuleLoader {
     specifier: &str,
     referrer: &str,
     _is_main: bool,
-  ) -> Result<ModuleSpecifier, AnyError>;
+  ) -> Result<ModuleSpecifier, Error>;
 
   /// Given ModuleSpecifier, load its source code.
   ///
@@ -99,7 +98,7 @@ pub trait ModuleLoader {
     _module_specifier: &ModuleSpecifier,
     _maybe_referrer: Option<String>,
     _is_dyn_import: bool,
-  ) -> Pin<Box<dyn Future<Output = Result<(), AnyError>>>> {
+  ) -> Pin<Box<dyn Future<Output = Result<(), Error>>>> {
     async { Ok(()) }.boxed_local()
   }
 }
@@ -114,7 +113,7 @@ impl ModuleLoader for NoopModuleLoader {
     _specifier: &str,
     _referrer: &str,
     _is_main: bool,
-  ) -> Result<ModuleSpecifier, AnyError> {
+  ) -> Result<ModuleSpecifier, Error> {
     Err(generic_error("Module loading is not supported"))
   }
 
@@ -142,7 +141,7 @@ impl ModuleLoader for FsModuleLoader {
     specifier: &str,
     referrer: &str,
     _is_main: bool,
-  ) -> Result<ModuleSpecifier, AnyError> {
+  ) -> Result<ModuleSpecifier, Error> {
     Ok(crate::resolve_import(specifier, referrer)?)
   }
 
@@ -257,7 +256,7 @@ impl RecursiveModuleLoad {
     load
   }
 
-  pub fn resolve_root(&self) -> Result<ModuleSpecifier, AnyError> {
+  pub fn resolve_root(&self) -> Result<ModuleSpecifier, Error> {
     match self.init {
       LoadInit::Main(ref specifier) => {
         self.loader.resolve(specifier, ".", true)
@@ -271,7 +270,7 @@ impl RecursiveModuleLoad {
     }
   }
 
-  pub async fn prepare(&self) -> Result<(), AnyError> {
+  pub async fn prepare(&self) -> Result<(), Error> {
     let op_state = self.op_state.clone();
     let (module_specifier, maybe_referrer) = match self.init {
       LoadInit::Main(ref specifier) => {
@@ -310,7 +309,7 @@ impl RecursiveModuleLoad {
     &mut self,
     scope: &mut v8::HandleScope,
     module_source: &ModuleSource,
-  ) -> Result<(), AnyError> {
+  ) -> Result<(), Error> {
     // Register the module in the module map unless it's already there. If the
     // specified URL and the "true" URL are different, register the alias.
     if module_source.module_url_specified != module_source.module_url_found {
@@ -392,7 +391,7 @@ impl RecursiveModuleLoad {
 }
 
 impl Stream for RecursiveModuleLoad {
-  type Item = Result<ModuleSource, AnyError>;
+  type Item = Result<ModuleSource, Error>;
 
   fn poll_next(
     self: Pin<&mut Self>,
@@ -529,7 +528,7 @@ impl ModuleMap {
     main: bool,
     name: &str,
     source: &str,
-  ) -> Result<ModuleId, AnyError> {
+  ) -> Result<ModuleId, Error> {
     let name_str = v8::String::new(scope, name).unwrap();
     let source_str = v8::String::new(scope, source).unwrap();
 
@@ -637,7 +636,7 @@ impl ModuleMap {
   pub async fn load_main(
     module_map_rc: Rc<RefCell<ModuleMap>>,
     specifier: &str,
-  ) -> Result<RecursiveModuleLoad, AnyError> {
+  ) -> Result<RecursiveModuleLoad, Error> {
     let load = RecursiveModuleLoad::main(specifier, module_map_rc.clone());
     load.prepare().await?;
     Ok(load)
@@ -646,7 +645,7 @@ impl ModuleMap {
   pub async fn load_side(
     module_map_rc: Rc<RefCell<ModuleMap>>,
     specifier: &str,
-  ) -> Result<RecursiveModuleLoad, AnyError> {
+  ) -> Result<RecursiveModuleLoad, Error> {
     let load = RecursiveModuleLoad::side(specifier, module_map_rc.clone());
     load.prepare().await?;
     Ok(load)
@@ -727,7 +726,6 @@ mod tests {
   use crate::RuntimeOptions;
   use futures::future::FutureExt;
   use parking_lot::Mutex;
-  use std::error::Error;
   use std::fmt;
   use std::future::Future;
   use std::io;
@@ -791,8 +789,8 @@ mod tests {
     }
   }
 
-  impl Error for MockError {
-    fn cause(&self) -> Option<&dyn Error> {
+  impl std::error::Error for MockError {
+    fn cause(&self) -> Option<&dyn std::error::Error> {
       unimplemented!()
     }
   }
@@ -803,7 +801,7 @@ mod tests {
   }
 
   impl Future for DelayedSourceCodeFuture {
-    type Output = Result<ModuleSource, AnyError>;
+    type Output = Result<ModuleSource, Error>;
 
     fn poll(self: Pin<&mut Self>, cx: &mut Context) -> Poll<Self::Output> {
       let inner = self.get_mut();
@@ -834,7 +832,7 @@ mod tests {
       specifier: &str,
       referrer: &str,
       _is_root: bool,
-    ) -> Result<ModuleSpecifier, AnyError> {
+    ) -> Result<ModuleSpecifier, Error> {
       let referrer = if referrer == "." {
         "file:///"
       } else {
@@ -978,7 +976,7 @@ mod tests {
         specifier: &str,
         referrer: &str,
         _is_main: bool,
-      ) -> Result<ModuleSpecifier, AnyError> {
+      ) -> Result<ModuleSpecifier, Error> {
         self.count.fetch_add(1, Ordering::Relaxed);
         assert_eq!(specifier, "./b.js");
         assert_eq!(referrer, "file:///a.js");
@@ -1096,7 +1094,7 @@ mod tests {
         specifier: &str,
         referrer: &str,
         _is_main: bool,
-      ) -> Result<ModuleSpecifier, AnyError> {
+      ) -> Result<ModuleSpecifier, Error> {
         self.count.fetch_add(1, Ordering::Relaxed);
         assert_eq!(specifier, "/foo.js");
         assert_eq!(referrer, "file:///dyn_import2.js");
@@ -1156,7 +1154,7 @@ mod tests {
       specifier: &str,
       referrer: &str,
       _is_main: bool,
-    ) -> Result<ModuleSpecifier, AnyError> {
+    ) -> Result<ModuleSpecifier, Error> {
       let c = self.resolve_count.fetch_add(1, Ordering::Relaxed);
       assert!(c < 7);
       assert_eq!(specifier, "./b.js");
@@ -1187,7 +1185,7 @@ mod tests {
       _module_specifier: &ModuleSpecifier,
       _maybe_referrer: Option<String>,
       _is_dyn_import: bool,
-    ) -> Pin<Box<dyn Future<Output = Result<(), AnyError>>>> {
+    ) -> Pin<Box<dyn Future<Output = Result<(), Error>>>> {
       self.prepare_load_count.fetch_add(1, Ordering::Relaxed);
       async { Ok(()) }.boxed_local()
     }
@@ -1292,7 +1290,7 @@ mod tests {
         specifier: &str,
         referrer: &str,
         _is_main: bool,
-      ) -> Result<ModuleSpecifier, AnyError> {
+      ) -> Result<ModuleSpecifier, Error> {
         self.resolve_count.fetch_add(1, Ordering::Relaxed);
         let s = crate::resolve_import(specifier, referrer).unwrap();
         Ok(s)
@@ -1634,7 +1632,7 @@ mod tests {
         specifier: &str,
         referrer: &str,
         _is_main: bool,
-      ) -> Result<ModuleSpecifier, AnyError> {
+      ) -> Result<ModuleSpecifier, Error> {
         let s = crate::resolve_import(specifier, referrer).unwrap();
         Ok(s)
       }
